/*
 * Created on Jun 11, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

import java.util.Iterator;
import java.util.Map;

/**
 * Static-method implementation of the Perl <code>chomp</code> keyword.
 * 
 * The <code>chomp</code> keyword is a safer version of <code>chop</code>
 * (see {@link Chop}) that only removes line-ending characters/sequences.
 * <p>
 * Like <code>chop</code>, <code>chomp</code> removes characters from the end
 * of a <code>StringBuffer</code> object. The return value from the operation
 * is different, however. In this case, the return value is the number of
 * characters deleted, rather than the specific characters themselves. When
 * operating on an array of <code>StringBuffer</code> objects, the value is
 * the total number of characters deleted from all buffers. Note that only
 * those character exactly matching the <code>line.separator</code> property
 * are removed (and only the very last characters, in cases where multiple
 * occurances are in a single buffer).
 * <p>
 * Here is an example, built around reading a file using the standard JDK
 * <code>BufferedReader</code> class for line-oriented data:
 * <pre>
 *     import java.io.*;
 *     import com.blackperl.Perl.Chomp;
 * 
 *     StringBuffer[] lines = new StringBuffer[1000];
 *     BufferedReader reader = new BufferedReader(new FileReader("lines"));
 *     String line = null; int index = 0;
 * 
 *     while ((line = reader.readLine()) != null)
 *     {
 *         lines[index] = new StringBuffer(line);
 *         Chomp.chomp(lines[index]);
 *         index++;
 *     }
 * 
 *     reader.close();
 * </pre>
 * <p>
 * Using the array form of <code>chomp</code>, the loop can be shortened to:
 * <pre>
 *     while ((line = reader.readLine()) != null)
 *         lines[index++] = new StringBuffer(line);
 *     Chomp.chomp(lines);
 * </pre>
 * The resulting content of <code>lines</code> would be the same.
 * <p>
 * To keep with the Perl idiom this emulates, the argument may also be an
 * object that implements the <code>Map</code> interface. If this is the case,
 * all the <i>values</i> (not keys) must be <code>StringBuffer</code> objects.
 * All will be chomped, just as with the array-form above (which also means
 * that the return value is the sum of all operations). This is the only form
 * of <code>chomp</code> that might throw a significant exception, in
 * the case where the value in a <code>Map</code> entry cannot be cast to
 * <code>StringBuffer</code>.
 * <p>
 * There are three modes of operation, based on the value of the record
 * separator when the method is called. In normal operation, when the separator
 * has a value that is not zero-length or null, that value is removed from the
 * end of the string (if present) exactly once. If the separator is a
 * zero-length (but not null) string, then <code>chomp</code> operates in what
 * Perl calls "paragraph mode". In this mode, all the trailing instances of
 * the "<code>\n</code>" character are removed (since the separator is empty,
 * it cannot be used as a guide). The string "<code>abc\n\n\n</code>" would be
 * chomped into "<code>abc</code>". Lastly, if the separator's value is
 * <code>null</code>, this is a mode that Perl calls "slurping". In that mode,
 * no chomping actually takes place. However many newline characters may be
 * at the end of the buffer, it remains unchanged. This is all controllable
 * through the <code>setRecordSeparator</code> method.
 * <p>
 * All components in this package provide an <code>instance</code> method to
 * retrieve a singleton object which may be used to call the static methods,
 * if the programmer prefers using an object to static invocation.
 * <p>
 * If this is the JDK 1.5 ("Tiger") edition of the package, this class is
 * suitable for use via static import:
 * <pre>
 *     // Using most of the code from the previous example, the changes are:
 *     import com.blackperl.Perl.Chomp.chomp;
 * 
 *     while ((line = reader.readLine()) != null)
 *         lines[index++] = new StringBuffer(line);
 *     chomp(lines);
 * </pre>
 * 
 * @author rjray
 * @version $Id: Chomp.java,v 1.11 2004/07/31 13:38:39 rjray Exp $
 */
public final class Chomp
{
    private static Chomp singleton;
    private static String RECORD_SEPARATOR;
    private static int RS_LENGTH;
    
    static
    {
        RECORD_SEPARATOR = System.getProperty("line.separator");
        RS_LENGTH = RECORD_SEPARATOR.length();
    }
    
    /**
     * Basic constructor, explicitly declared here as <code>private</code> to
     * prevent other classes instantiating this class.
     */
    private Chomp()
    {
        super();
    }
        
    /**
     * The <code>instance</code> method is used to retrieve the
     * <code>Chomp</code> singleton that applications can use in lieu of
     * invoking the methods statically.
     * 
     * @return  Singleton instance of the Chomp class
     */
    public synchronized static Chomp instance()
    {
        if (singleton == null)
        {
            singleton = new Chomp();
        }
        
        return(singleton);
    }

    /**
     * Assign a new record-separator to be used by the
     * <code>chomp</code> variants.
     * 
     * @param newRS A string to be used as the record-separator (line-end)
     */
    public static void setRecordSeparator(String newRS)
    {
        RECORD_SEPARATOR = newRS;
        RS_LENGTH = (newRS == null) ? 0 : RECORD_SEPARATOR.length();
    }
    
    /**
     * Retrieve the current value of the record-separator (line-end sequence).
     * 
     * @return The current record-separator (line-terminator)
     */
    public static String getRecordSeparator()
    {
        return RECORD_SEPARATOR;
    }
    
    /**
     * Perform a <code>chomp</code> on the <code>StringBuffer</code> passed in
     * as a parameter.
     * 
     * @param buffer    StringBuffer with the string data to trim
     * @return  The number of characters removed
     */
    public static int chomp(StringBuffer buffer)
    {
        int retval = 0;
        
        if (RECORD_SEPARATOR != null)
        {
            if (RS_LENGTH == 0)
            {
                retval = chompParaMode(buffer);
            }
            else
            {
                retval = chompLineMode(buffer, RECORD_SEPARATOR, RS_LENGTH);
            }
        }
        
        return retval;
    }
    
    /**
     * Do a <code>chomp</code> on an array of <code>StringBuffer</code>
     * objects. Returns the total number of characters removed from all buffers.
     * 
     * @param buffers   An array of StringBuffer objects to operate upon
     * @return  The total number of characters removed from all buffers
     */
    public static int chomp(StringBuffer[] buffers)
    {
        int retval = 0;
        
        // The test of null-ness here is replicated across all chomp forms
        // because it is more efficient to test it here and bypass the array
        // iteration when it is null.
        if (RECORD_SEPARATOR != null)
        {
            // Again, we are replicating the loop for the sake of not
            // repeatedly testing RS_LENGTH
            if (RS_LENGTH == 0)
            {
                for (int idx = 0; idx < buffers.length; idx++)
                {
                    retval += chompParaMode(buffers[idx]);
                }
            }
            else
            {
                for (int idx = 0; idx < buffers.length; idx++)
                {
                    retval += chompLineMode(buffers[idx],
                        RECORD_SEPARATOR, RS_LENGTH);
                }
            }
        }
        
        return retval;
    }
    
    /**
     * Perform a <code>chomp</code> on the values of the <code>Map</code>
     * object passed in. Returns the total number of characters removed.
     * 
     * @param table An object implementing the Map interface
     * @return  The character from the last chop().
     * @throws ClassCastException
     */
    public static int chomp(Map table) throws ClassCastException
    {
        int retval = 0;
        StringBuffer obj;
        
        // The test of null-ness here is replicated across all chomp forms
        // because it is more efficient to test it here and bypass the array
        // iteration when it is null.
        if (RECORD_SEPARATOR != null)
        {
            // Again, we are replicating the loop for the sake of not
            // repeatedly testing RS_LENGTH
            if (RS_LENGTH == 0)
            {
                for (Iterator iter = table.values().iterator(); iter.hasNext(); )
                {
                    obj = (StringBuffer)iter.next();
                    retval += chompParaMode(obj);
                }
            }
            else
            {
                for (Iterator iter = table.values().iterator(); iter.hasNext(); )
                {
                    obj = (StringBuffer)iter.next();
                    retval += chompLineMode(obj, RECORD_SEPARATOR, RS_LENGTH);
                }
            }
        }
        
        return retval;
    }
    
    /**
     * Chomp a single StringBuffer in line-mode, taking just the first
     * occurance of the separator string. Return number of characters dropped.
     * 
     * @param buffer    StringBuffer object to chomp
     * @param sep       String separator value
     * @param len       Length of the <code>sep</code> parameter
     * @return      Total number of characters removed
     */
    private static int chompLineMode(StringBuffer buffer, String sep, int len)
    {
        boolean found = false;
        
        if ((buffer.length() > 0) &&
            (buffer.substring(buffer.length() - len).equals(sep)))
        {
            found = true;
            buffer.delete((buffer.length() - len), buffer.length());
        }
        
        return (found ? len : 0);
    }
    
    /**
     * Chomp the buffer in paragraph mode, which is to say keep deleting as
     * long as there are "\n" characters at the end of the buffer. Uses the
     * constant "\n" instead of RECORD_SEPARATOR, per Perl design.
     * 
     * @param buffer    StringBuffer to remove newlines from
     * @return      The total number of characters removed
     */
    private static int chompParaMode(StringBuffer buffer)
    {
        int total = 0;
        
        if (buffer.length() > 0)
        {
            while (buffer.substring(buffer.length() - 1).equals("\n"))
            {
                total++;
                buffer.delete((buffer.length() - 1), buffer.length());
            }
        }
        
        return total;
    }
}
