/*
 * Created on Jun 23, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

import junit.framework.TestCase;
import java.util.HashMap;

/**
 * JUnit class for testing the Chomp class
 * 
 * This tests the various methods made available by the <code>Chomp</code>
 * class. There are three utility methods and two signature-forms of the
 * basic <code>chomp</code> operation itself.
 * 
 * @author rjray
 * @version $Id: ChompTest.java,v 1.5 2004/07/31 13:38:39 rjray Exp $
 */
public class ChompTest extends TestCase
{
    private static String lineSep;

    /**
     * Constructor for ChompTest.
     * @param name Identifying string for the test
     */
    public ChompTest(String name)
    {
        super(name);
    }
    
    // Stuff to do before any tests are run
    protected void setUp()
    {
        lineSep = System.getProperty("line.separator", "\n");
    }

    /*
     * Test the instance() method to ensure that it always returns the same
     * object.
     */
    final public void testInstance()
    {
        // Call instance() twice. See if they match.
        Chomp c1 = Chomp.instance();
        Chomp c2 = Chomp.instance();
        
        assertEquals("Test two calls to instance() for equality", c1, c2);
    }

    final public void testGetRecordSeparator()
    {
        assertEquals("Testing getRecordSeparator",
            lineSep, Chomp.getRecordSeparator());
    }

    final public void testSetRecordSeparator()
    {
        Chomp.setRecordSeparator("//");
        assertEquals("Testing setRecordSeparator",
            "//", Chomp.getRecordSeparator());
        Chomp.setRecordSeparator(lineSep);
    }

    /*
     * Test for int chomp(StringBuffer)
     */
    final public void testChompStringBuffer()
    {
        StringBuffer testBuf = new StringBuffer(20);
        int count;
        
        testBuf.append("text");
        testBuf.append(lineSep);
        
        count = Chomp.chomp(testBuf);
        assertEquals("Testing chomp(StringBuffer)",
            "text", testBuf.toString());
        assertEquals("Testing chomp(StringBuffer) - return value",
            count, lineSep.length());
    }

    /*
     * Test for int chomp(StringBuffer[])
     */
    final public void testChompStringBufferArray()
    {
        String[] strings = { "one", "two", "three", "four" };
        StringBuffer[] testBuffArr = new StringBuffer[strings.length];
        int count, times = 0, i;
        
        for (i = 0; i < strings.length; i++)
        {
            testBuffArr[i] = new StringBuffer(strings[i]);
            if ((i & 1) == 0)
            {
                testBuffArr[i].append(lineSep);
                times++;
            } 
        }
        
        count = Chomp.chomp(testBuffArr);
        assertEquals("Testing chomp(StringBuffer[]) - return value",
            count, (lineSep.length() * times));
        for (i = 0; i < strings.length; i++)
        {
            assertEquals("Testing chomp(StringBuffer[" + i + "]",
                strings[i], testBuffArr[i].toString());
        }
    }
    
    final public void testChompMap()
    {
        String[] strings = { "uno", "dos", "tres", "quatro" };
        StringBuffer[] testBufArr = new StringBuffer[strings.length];
        HashMap map = new HashMap();
        int count, times = 0, i;
        
        for (i = 0; i < strings.length; i++)
        {
            testBufArr[i] = new StringBuffer(strings[i]);
            if ((i & 1) == 0)
            {
                testBufArr[i].append(lineSep);
                times++;
            }
            map.put(new Integer(i), testBufArr[i]);
        }
        
        count = Chomp.chomp(map);
        assertEquals("Testing chomp(Map) - return value",
            count, (lineSep.length() * times));
        // This is a cheat, since the Map had the same pointers. It's easier
        // than converting the Map's key back to int to use as an index.
        for (i = 0; i < strings.length; i++)
        {
            assertEquals("Testing chomp(Map via StringBuffer[" + i + "])",
                strings[i], testBufArr[i].toString());
        }
    }
    
    /*
     * Test chomp with a zero-length record-separator ("paragraph" mode)
     */
    final public void testChompZeroSep()
    {
        String oldSep = Chomp.getRecordSeparator();
        Chomp.setRecordSeparator("");
        StringBuffer simple = new StringBuffer("simple\n\n");
        
        int chomped = Chomp.chomp(simple);
        // First, (currently) nothing should have been eaten
        assertTrue("Testing chomp(String) with zero-length RS, int value",
            chomped == ("\n".length() * 2));
        // Check that simple is what we expect it to be
        assertTrue("Testing chomp(String) with zero-length RS, buffer value",
            simple.toString().equals("simple"));
        
        Chomp.setRecordSeparator(oldSep);
    }
    
    /*
     * Test chomp with the record-separator set to null ("slurp" mode)
     */
    final public void testChompNullSep()
    {
        String oldSep = Chomp.getRecordSeparator();
        Chomp.setRecordSeparator(null);
        StringBuffer simple = new StringBuffer("null separator\n\n");
        
        int chomped = Chomp.chomp(simple);
        // First, (currently) nothing should have been eaten
        assertTrue("Testing chomp(String) with null RS, int value",
            chomped == 0);
        // Check that simple is what we expect it to be
        assertTrue("Testing chomp(String) with null RS, buffer value",
            simple.toString().equals("null separator\n\n"));
        
        Chomp.setRecordSeparator(oldSep);
    }
}
