/*
 * Created on Jun 11, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

import java.util.regex.Pattern;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * Static-method implementation of the Perl <code>grep</code> keyword.
 * 
 * The <code>grep</code> function traverses a <code>List</code> (or array) of
 * objects. Each of the objects are evaluated using the <code>eval</code>
 * method of the block passed in (see {@link GrepBlock}). When the method
 * returns a <code>Boolean</code> object as a result, and the result's value
 * is "true", the tested object is added to the list of values to be returned
 * by <code>grep</code>. If <code>eval</code> returns an object other than
 * <code>Boolean</code>, and the value is not <code>null</code>, the value
 * returned by <code>eval</code> is what is put into the list of return values.
 * <p>
 * Because many of the common uses of <code>grep</code> in Perl involve
 * testing against string values or regular expressions, there are also some
 * convenience forms of the function provided here, to make coding a little
 * easier. See the method documentation for details.
 * <p>
 * The "default" forms of <code>grep</code> take <code>List</code> objects as
 * the parameter to provide the set of values to test. Any of the classes that
 * implement the <code>List</code> interface may be used. Alternate forms of
 * the function are also provided that take and return ordinary arrays of
 * <code>Object</code>.
 * <p>
 * Here are some simple examples of <code>grep</code>:
 * <pre>
 *     import java.util.*;
 *     import com.blackperl.Perl.Grep;
 * 
 *     Vector v = new Vector();
 *     for (int i = 0; i < 100; i++)
 *         v.add(new Integer(i));
 *     List even_numbers = Grep.grep(new GrepBlock() {
 *             public Object eval(Object I) {
 *                 return Boolean.valueOf((((Integer)I).intValue() & 1) == 0);
 *             } }, v);
 *     // even_numbers now has all even numbers in the range 0 to 99
 * </pre>
 * <p>
 * Using arrays instead of the <code>List</code> interface:
 * <p>
 * <pre>
 *     import java.util.regex.*;
 *     import com.blackperl.Perl.Grep;
 * 
 *     String[] languages = { "Java (vm)", "Perl (scripting)", "C (compiled)",
 *         "Python (scripting)", "Ruby (scripting)", "C# (vm)",
 *         "Fortran (compiled)" };
 * 
 *     // Returns Java and C#
 *     Object[] vm_languages = Grep.grep("(vm)", languages);
 * 
 *     // Returns Perl and Python
 *     Object[] p_languages = Grep.grep(Pattern.compile("^P"), languages);
 * </pre>
 * Note that while the array interfaces may be easier to use in many cases
 * that the <code>List</code> interfaces, the return value is always of the
 * <code>Object[]</code> form-- trying to cast the return value in the above
 * examples so as to assign directly to an array of <code>String</code> would
 * raise a <code>ClassCastException</code>. The choice of <code>List</code>
 * over array is largely a matter of choosing when and where you want to
 * cast your result objects. Eventually, the JDK 1.5 version of this class
 * will utilize generics to reduce the complexity of this.
 * <p>
 * All components in this package provide an <code>instance</code> method to
 * retrieve a singleton object which may be used to call the static methods,
 * if the programmer prefers using an object to static invocation.
 * <p>
 * If this is the JDK 1.5 ("Tiger") edition of the package, this class is
 * suitable for use via static import:
 * <pre>
 *     import com.blackperl.Perl.Grep.grep;
 * 
 *     // Assume the same "languages" string array as previous examples
 * 
 *     // Returns Perl, Python and Ruby.
 *     String[] scripting_languages = grep("(scripting)", languages);
 * </pre> 
 * 
 * @author rjray
 * @version $Id: Grep.java,v 1.7 2004/07/21 09:23:59 rjray Exp $
 */
public class Grep
{
    private static Grep singleton;
    
    /**
     * Basic constructor, explicitly declared here as <code>private</code> to
     * prevent other classes instantiating this class.
     */
    private Grep()
    {
        super();
    }
    
    /**
     * The <code>instance</code> method is used to retrieve the
     * <code>Grep</code> singleton that applications can use in lieu of
     * invoking the methods statically.
     * 
     * @return  Singleton instance of the Chomp class
     */
    public synchronized static Grep instance()
    {
        if (singleton == null)
        {
            singleton = new Grep();
        }
        
        return(singleton);
    }
    
    /*
     * This is a very small, private class used by one the shortcut-style forms
     * of grep. It takes a pre-compiled java.util.regex.Pattern object and
     * constructs an eval() that will match the pattern against the test
     * object.
     */
    private final class RegexpBlock implements GrepBlock
    {
        private Pattern pat;
        
        protected RegexpBlock(final Pattern p)
        {
            pat = p;
        }
            
        public Object eval(final Object o)
        {
            return Boolean.valueOf(pat.matcher(o.toString()).find());
        }
    }

    /*
     * This is another small, private class. Like the regex-based one above,
     * this is just for use by some of the convenience forms of grep. This one
     * constructs an eval() to look for the given string as a substring of the
     * test object.
     */
    private final class StringBlock implements GrepBlock
    {
        private String str;
            
        protected StringBlock(final String s)
        {
            str = s;
        }
            
        public Object eval(final Object o)
        {
            return Boolean.valueOf(o.toString().indexOf(str) != -1);
        }
    }
        
    /**
     * Do a grep over the list of objects, evaluating each one via the given
     * <code>GrepBlock</code> (see {@link GrepBlock}). When the
     * <code>eval</code> method is executed, the result is interpreted in one
     * of two ways: If the return value is a <code>Boolean</code> object, and
     * that object has a value of "true", the original object (from
     * <code>list</code> is inserted into the return-list. If the return value
     * is an object other than <code>Boolean</code> (or derived from), it is
     * added to the return-list. If the return value is either <code>null</code>
     * or a <code>Boolean</code> with value "false", then no value is added to
     * the return.
     * 
     * @param block An object of a class implementing the GrepBlock interface\
     * @param list  A List object with all the values to test via the block
     * @return  A List object of all the matched values
     */
    public static List grep(final GrepBlock block, final List list)
    {
        Vector retval = new Vector(list.size());
        
        for (Iterator iter = list.iterator(); iter.hasNext(); )
        {
            Object saved = iter.next();
            Object o = block.eval(saved);
            
            if (o != null)
            {
                // If the return value is a Boolean object and the value of it
                // is "true", add the actual list element itself.
                if (o instanceof Boolean)
                {
                    if (((Boolean)o).booleanValue())
                    {
                        retval.add(saved);
                    } 
                }
                // Otherwise, for all other (non-null) values, add the return
                // value from the evaluation function.
                else
                {
                    retval.add(o);
                }
            }
        }
        
        return retval;
    }
    
    /**
     * This is a shorthand-form for <code>grep</code>, that takes a
     * <code>java.util.regex.Pattern</code> object and used an internal class
     * that implements <code>GrepBlock</code> to provide an evaluation
     * function that returns a <code>Boolean</code> value indicating whether
     * the stringification of the given test object matched the pattern. Those
     * that match are collected and returned in a <code>List</code> object as
     * the return value. This assumes that for all the objects in
     * <code>list</code> the <code>toString()</code> method is callable.
     * 
     * @param pattern   The regular expression to test objects against
     * @param list  A List object containing the objects to test against it
     * @return  A List object containing all the objects that matched the
     * pattern
     */
    public static List grep(final Pattern pattern, final List list)
    {
        return grep(instance().new RegexpBlock(pattern), list);
    }
    
    /**
     * This is another shorthand-form for <code>grep</code>, this one taking a
     * <code>String</code> object and constructing a <code>GrepBlock</code>
     * that tests the string to see if it is a contained substring within the
     * stringification of each test object. Those that contain
     * <code>string</code> as a substring are added to the output list. This
     * assumes that all the objects in <code>list</code> can call a
     * <code>toString</code> method.
     * 
     * @param string    The String to test as a substring in the objects
     * @param list  A List object containing the objects to test against
     * @return  A List object containing all the matching objects
     */
    public static List grep(final String string, final List list)
    {
        return grep(instance().new StringBlock(string), list);
    }
    
    /**
     * This is identical to the block-list form of <code>grep</code>, except
     * that it takes and returns an array of <code>Object</code>, rather than
     * a <code>List</code> instance.
     * 
     * @param block The GrepBlock to evaluate objects against
     * @param objArray  An array of Objects to be evaluated
     * @return  An array of those values matched by the block evaluation
     */
    public static Object[] grep(final GrepBlock block, final Object[] objArray)
    {
        return (grep(block, new Vector(Arrays.asList(objArray)))).toArray();
    }
    
    /**
     * The pattern-list form of <code>grep</code>, only taking and returning
     * arrays of Object, rather than <code>List</code> instances.
     * 
     * @param pattern   The Pattern to compare each object against
     * @param objArray  The array of Objects to test
     * @return  An array of those Objects that matched
     */
    public static Object[] grep(final Pattern pattern, final Object[] objArray)
    {
        return grep(instance().new RegexpBlock(pattern), objArray);
    }
    
    /**
     * The string-list form of <code>grep</code>, with arrays instead of the
     * <code>List</code> instances.
     * 
     * @param string    The String to compare each object with
     * @param objArray  The array of Objects to test
     * @return  An array of those Objects that matched
     */
    public static Object[] grep(final String string, final Object[] objArray)
    {
        return grep(instance().new StringBlock(string), objArray);
    }
}
