/*
 * Created on Jun 10, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

/**
 * Static-method implementation of the Perl <code>join</code> keyword.
 * 
 * This class makes available as static methods various forms of the
 * <code>join</code> functionality. All forms accomplish the same thing: they
 * return a <code>String</code> object that is the concatenation of the list
 * of strings, using the token as a separator in between. The variations are
 * only to permit flexibility in how the separator is specified.
 * <p>
 * For example, if the array <code>stringArray</code> contained the strings:
 * "one", "two", "three" and "four", then the following:
 * <p>
 * <pre>
 *     import com.blackperl.Perl.Join;
 * 
 *     String joined = Join.join(" and a ", stringArray);
 * </pre>
 * <p>Would yield the string:
 * <pre>
 *     one and a two and a three and a four
 * </pre>
 * <p>
 * Since the sequence used for joining is often a single character, there is a
 * form that allows the first argument to be of type <code>char</code>. If the
 * same array were used for this:
 * <p>
 * <pre>
 *     import com.blackperl.Perl.Join;
 * 
 *     String joined = Join.join(',', stringArray);
 * </pre>
 * <p>
 * The result would be:
 * <p>
 * <pre>
 *     one,two,three,four
 * </pre>
 * <p>
 * If the array passed to <code>join</code> is empty or null, then an empty
 * (zero-length) string value is returned.
 * <p>
 * All components in this package provide an <code>instance</code> method to
 * retrieve a singleton object which may be used to call the static methods,
 * if the programmer prefers using an object to static invocation.
 * <p>
 * If this is the JDK 1.5 ("Tiger") edition of the package, this class is
 * suitable for use via static import:
 * <pre>
 *     import com.blackperl.Perl.Join.join;
 * 
 *     String joined = join(',', stringArray);
 *     // Same result as the previous example
 * </pre>
 * 
 * @author rjray
 * @version $Id: Join.java,v 1.12 2004/07/31 12:27:07 rjray Exp $
 */
public final class Join
{
    private static Join singleton;
    
    /**
     * Basic constructor, explicitly declared here as <code>private</code> to
     * prevent other classes instantiating this class.
     */
    private Join()
    {
        super();
    }
    
    /**
     * The <code>instance</code> method is used to retrieve the
     * <code>Join</code> singleton that applications can use in lieu of
     * invoking the methods statically.
     * 
     * @return  Singleton instance of the Chomp class
     */
    public synchronized static Join instance()
    {
        if (singleton == null)
        {
            singleton = new Join();
        }
        
        return(singleton);
    }

    /**
     * Create a <code>String</code> by concatenating the items in the array
     * together, with an instance of the separator between each.
     * 
     * @param token A single character to use as the separator
     * @param array  Array of <code>String</code> objects to join together
     * @return  The crafted <code>String</code>
     */
    public static String join(char token, String[] array)
    {
        StringBuffer ntoken = new StringBuffer(1);
        ntoken.append(token);
        return(join(ntoken.toString(), array));    
    }
    
    /**
     * Create a <code>String</code> by concatenating the items in the array
     * together, with an instance of the separator between each.
     * 
     * @param token The <code>String</code> to use as the separator
     * @param array  Array of <code>String</code> objects to join together
     * @return  The crafted <code>String</code>
     */
    public static String join(String token, String[] array)
    {
        // Start with a default size of 1K
        StringBuffer result = new StringBuffer(1024);
        
        // If the list object is null, the default will be to return a null
        // string. If the list is empty, we'll also return the null string
        if ((array != null) && (array.length > 0))
        {
            // Start with the first one
            result.append(array[0]);
            
            // Assuming there are others, iterate over them
            if (array.length > 1)
            {
                // If token is null or zero-length, loop without it
                if (token == null || token.length() == 0)
                {
                    for (int i = 1; i < array.length; i++)
                    {
                        result.append(array[i]);
                    }
                }
                else
                {
                    for (int i = 1; i < array.length; i++)
                    {
                        result.append(token);
                        result.append(array[i]);
                    }
                }
            }
        }
        return result.toString();
    }
}
