/*
 * Created on Jun 10, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * Static-method implementation of the Perl <code>map</code> keyword.
 * 
 * The <code>map</code> function iterates over a list of values, applying a
 * function to each one in turn. It is similar in this aspect to
 * <code>grep</code> (see {@link Grep}), except that <code>map</code> is more
 * about the evaluation of the provided function, and less about selecting
 * a sub-set of the input list.
 * <p>
 * <code>map</code> takes a block to execute-- one of {@link MapBlock} or
 * {@link GrepBlock}-- and gathers together the values that the
 * <code>eval()</code> method of the block returns. Thus, unlike
 * <code>grep</code>, <code>map</code> may produce more elements than it is
 * fed as input. In Perl, this is often used for such tasks as generating
 * associative arrays (the analog of <code>java.util.HashTable</code>) or
 * applying list-oriented processing to a set of input values as part of a
 * larger pipeline (with the output often going to another <code>map</code>, or
 * possibly a <code>grep</code>).
 * <p>
 * Unfortunately, things like creating hash tables on the fly are not as easy
 * in Java, as there is no clear way in the foundation classes to promote a
 * <code>List</code> instance into a <code>Map</code>, which is the base
 * interface needed to instantiate classes such as
 * <code>java.util.HashTable</code> or <code>java.util.Hashmap</code> from
 * existing data. Instead, the developer would use the functionality of the
 * <code>MapBlock</code> or <code>GrepBlock</code> in the form of a named
 * (usually <b>private</b>) inner-class with accessors to build up a
 * <code>Map</code> from the input objects, then retrieve that data through the
 * accessors after <code>map</code> has completed. One of the examples below
 * illustrates this concept.
 * <p>
 * This first example shows very basic usage of <code>map</code>, applying the
 * <code>split</code> method of the <code>String</code> class to the passed-in
 * elements and producing an arbitrary number of <code>String</code> objects
 * as a result:
 * <pre>
 *     import com.blackperl.Perl.Map;
 *     import com.blackperl.Perl.MapBlock;
 * 
 *     // Assume "lines" is of type String[]
 *     Object[] all = Map.map(new MapBlock() {
 *         public Object[] eval(Object o) {
 *             return ((String)o).split(",");
 *         } }, lines);
 *     // "all" is now an array of strings all split up by commas
 * </pre>
 * This example used arrays rather than <code>List</code> instances, and it
 * suffers from the same limitations seen in {@link Grep}. That is, the
 * returned array is always of type <code>Object[]</code>, and thus the
 * elements have to be explicitly cast when they are later referenced.
 * <p>
 * A common use of <code>map</code> in Perl is to turn an array of strings
 * into a testable associative array, so that further input values can be
 * tested against the list without having to explicitly traverse the list
 * for every test. This next example shows how this can be done:
 * <pre>
 *     import com.blackperl.Perl.Map;
 *     import com.blackperl.Perl.MapBlock;
 *     import java.util.HashMap;
 *     import java.util.List;
 * 
 *     class MapMaker implements MapBlock
 *     {
 *         private java.util.Map internalMap;
 * 
 *         public MapMaker(java.util.Map map) { internalMap = map; }
 * 
 *         Object[] eval(Object o)
 *         {
 *             internalMap.put((String)o, Boolean.valueOf(true));
 *             return null;
 *         }
 *     }
 * 
 *     // An instance field, which wouldn't otherwise be accessible to a
 *     // MapBlock instance
 *     HashMap booleanWordMap = new HashMap();
 * 
 *     // Assume "words" is set up elsewhere as some implementation of List
 *     Map.map(new MapMaker(booleanWordMap), words);
 *     // The booleanWordMap hash table now has each string that was
 *     // contained within the List called words as a key. The value for
 *     // each of these keys is the Boolean object equivalent of "true".
 * </pre>
 * One could have the <code>MapMaker</code> code count the words, rather than
 * just map them to <code>true</code>, depending on the needs of the program
 * itself.
 * <p>
 * In cases where it is known that the mapping function is only going to
 * produce at most one output element per input element, the {@link GrepBlock}
 * interface may be used in place of {@link MapBlock}. This makes it a little
 * easier to write the <code>eval</code> hook cleanly, without the need to
 * create and return an array when only one value is being passed. The next
 * example shows this (while also using the {@link Ucfirst} component):
 * <pre>
 *     import com.blackperl.Perl.Map;
 *     import com.blackperl.Perl.GrepBlock;
 *     import com.blackperl.Perl.Ucfirst;
 *     import java.util.List;
 * 
 *     // Assume again that "words" is some List object set up elsewhere
 *     List capitalizedWords = Map.map(new GrepBlock() {
 *         Object eval(Object word) {
 *             return Ucfirst.ucfirst(((String)word).toLowerCase());
 *         } }, words);
 * </pre>
 * When this has run, the <code>List</code> object <code>capitalizedWords</code>
 * contains all the strings from <code>words</code> in the same order. Each of
 * these new strings has been capitalized by first shifting the whole string
 * to lower-case with the <code>toLowerCase</code> method, then passing that
 * result to {@link Ucfirst#ucfirst(java.lang.String)}.
 * In other words, this code recreates a common Perl idiom:
 * <pre>
 *     &#64;capitalized_words = map { ucfirst lc $_ } &#64;words;
 * </pre>
 * <p>
 * All components in this package provide an <code>instance</code> method to
 * retrieve a singleton object which may be used to call the static methods,
 * if the programmer prefers using an object to static invocation.
 * <p>
 * If this is the JDK 1.5 ("Tiger") edition of the package, this class is
 * suitable for use via static import. The invocation from the second
 * example could instead be expressed as (with the changed <code>import</code>
 * line included):
 * <pre>
 *     import com.blackperl.Perl.Map.map;
 * 
 *     map(new MapMaker(booleanWordMap), words);
 * </pre>
 * 
 * @author rjray
 * @version $Id: Map.java,v 1.5 2004/07/21 09:23:59 rjray Exp $
 */
public class Map
{
    private static Map singleton;
    
    /**
     * Basic constructor, explicitly declared here as <code>private</code> to
     * prevent other classes instantiating this class.
     */
    private Map()
    {
        super();
    }
    
    /**
     * The <code>instance</code> method is used to retrieve the
     * <code>Map</code> singleton that applications can use in lieu of
     * invoking the methods statically.
     * 
     * @return  Singleton instance of the Map class
     */
    public synchronized static Map instance()
    {
        if (singleton == null)
        {
            singleton = new Map();
        }
        
        return(singleton);
    }
    
    /**
     * Map the functionality specified by <code>eval()</code> in the
     * {@link MapBlock} instance over the objects contained in the
     * <code>List</code>, returning all the values produced as a result.
     * 
     * @param block The MapBlock instance providing the eval() method
     * @param list  A List instance providing the objects to be evaluated
     * @return  A List instance containing the values returned in the
     * evaluation of the values in <code>list</code>
     */
    public static List map(MapBlock block, List list)
    {
        Vector retval = new Vector(list.size());
        Object[] evalReturn;
        
        for (Iterator iter = list.iterator(); iter.hasNext(); )
        {
            Object o = iter.next();
            evalReturn = block.eval(o);
            
            if (evalReturn != null)
            {
                for (int i = 0; i < evalReturn.length; i++)
                {
                    retval.add(evalReturn[i]);
                }
            }
        }
        
        return retval;
    }
    
    /**
     * Perform <code>map</code> using a {@link MapBlock} interface and an
     * array of objects (rather than a List instance).
     * 
     * @param block The MapBlock instance providing the eval() method
     * @param objArray  The array of Object to evaluate in sequence
     * @return  An array (possibly empty) of the objects produced by eval()
     */
    public static Object[] map(MapBlock block, Object[] objArray)
    {
        return (map(block, new Vector(Arrays.asList(objArray)))).toArray();
    }
    
    /**
     * Map the functionality specified by <code>eval()</code> in the
     * {@link GrepBlock} instance over the objects contained in the
     * <code>List</code>, returning all the values produced as a result.
     * 
     * @param block The GrepBlock instance that provides eval()
     * @param list  The List instance with objects to evaluate in sequence
     * @return  A List instance containing the values returned in the
     * evaluation of the values in <code>list</code>
     */
    public static List map(GrepBlock block, List list)
    {
        Vector retval = new Vector(list.size());
        Object evalReturn;
        
        for (Iterator iter = list.iterator(); iter.hasNext(); )
        {
            Object o = iter.next();
            evalReturn = block.eval(o);
            
            // This is easier than the MapBlock, since there is just one value
            if (evalReturn != null)
            {
                retval.add(evalReturn);
            }
        }
        
        return retval;
    }
    
    /**
     * Perform <code>map</code> using a {@link GrepBlock} interface and an
     * array of objects (rather than a List instance).
     * 
     * @param block The GrepBlock instance that provides eval()
     * @param objArray  The array of Object to evaluate in sequence
     * @return  An array (possibly empty) of the objects produced by eval()
     */
    public static Object[] map(GrepBlock block, Object[] objArray)
    {
        return (map(block, new Vector(Arrays.asList(objArray)))).toArray();
    }
}
