/*
 * Created on Jul 29, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

/**
 * A class for exceptions thrown by the <code>Tr</code> class.
 * 
 * In some cases, exceptions from within the <code>tr</code> method may want
 * to attach the pattern that was being analyzed when the exception occurred.
 * For these cases, this class defines additional constructors (besides the
 * ones inherited from <code>java.lang.RuntimeException</code>) that allow
 * specifying the offending pattern. Receivers of exceptions constructed from
 * this class may then use the <code>getBadPattern</code> method to later
 * retrieve the pattern.
 * <p>
 * Using this approach, the message of the exception object can still be used
 * to describe the exact nature of the error, and receivers do not have to
 * resort to string-parsing or other manipulations to pull the pattern out of
 * the message.
 * <p>
 * The one limitation here is that it is not possibly to create an exception
 * object of this class with <i>just</i> the bad pattern. Because this
 * signature conflicts with an existing constructor signature from the parent
 * class, any attempt to set the pattern during initialization must use the
 * forms documented here, in which at least one other parameter is also
 * provided.
 * 
 * @author rjray
 * @version $Id$
 */
public final class TrException extends RuntimeException
{
    private final String badPattern;
    
    /**
     * Create an exception with the specified error message and offending
     * <code>tr</code> pattern.
     * 
     * @param message   The message of the exception
     * @param pattern   The offending pattern string
     */
    public TrException(String message, String pattern)
    {
        super(message);
        
        badPattern = pattern;
    }
    /**
     * Create an exception with the specified error message, using the given
     * <code>Throwable</code> instance as the cause, and the given
     * <code>tr</code> pattern.
     * 
     * @param message   The message of the exception
     * @param cause     The lower-level exception that led to this
     * @param pattern   The offending pattern string
     */
    public TrException(String message, Throwable cause, String pattern)
    {
        super(message, cause);
        
        badPattern = pattern;
    }
    /**
     * Create an exception using the given <code>Throwable</code> instance as
     * the cause, and the given <code>tr</code> pattern.
     * 
     * @param cause     The lower-level exception that led to this
     * @param pattern   The offending pattern string
     */
    public TrException(Throwable cause, String pattern)
    {
        super(cause);
        
        badPattern = pattern;
    }
    
    
    /**
     * Retrieve the offending pattern for this instance. Note that once set,
     * the pattern for an exception of this class becomes static and may not
     * be modified.
     * 
     * @return Returns the badPattern.
     */
    public String getBadPattern()
    {
        return badPattern;
    }
}
